`include "xgriscv_defines.v"

module datapath(
	input						clk, reset,

	input [`INSTR_SIZE-1:0]		instrIF,
	output[`ADDR_SIZE-1:0]		pcIF,

	input [`XLEN-1:0]			readdataMEM, 	// from data memory: read data
	output[`XLEN-1:0]			aluoutMEM,		// to data memory: address
 	output[`XLEN-1:0]			wmemdataMEM, 	    // to data memory: write data
	output						memwriteMEM,	// to data memory: write enable
	output [`ADDR_SIZE-1:0]     pcMEM, pcWB, 
	
	// from controller
	input [4:0]		            immctrlID,
	input			            itype, jalID, jalr, bunsigned, pcsrcID,
	input [3:0]		            aluctrlID,
	input [1:0]		            alusrcaID,
	input			            alusrcbID,
	input						memwriteID, lunsignedID,
	input          		        memtoregID, regwriteID,
	input [1:0]                 lwhbID, swhbID,
	
  	// to controller
	output [6:0]				opID,
	output [2:0]				funct3ID,
	output [6:0]				funct7ID,
	output [4:0]				rdID, rs1ID, 
	output [3:0]                ampMEM,
	output [11:0]  		        immID,
	output 	       		        zero, lt
	);

	//声明
	wire [`ADDR_SIZE-1:0]   base, offset, offsetID;
    wire                    stallIF = 1'b0;
	wire					stallID = 1'b0;
	wire                    flushID = 1'b0; 
	wire 					flushEX = 1'b0;
	wire					flushMEM= 1'b0;
	wire					flushWB = 1'b0;
	wire [`ADDR_SIZE-1:0]   newpcIF, pcplus4IF, pcbranchID;
	wire [4:0]              shamtID, shamtEX;
	wire [`ADDR_SIZE-1:0]   instrID, pcplus4ID, pcplus4EX, pcplus4MEM, pcplus4IN;
	wire [`RFIDX_WIDTH-1:0] rs2ID;
	wire [`XLEN-1:0]		immoutID, immoutEX, shftimmID, shftimmEX;
	wire [`XLEN-1:0]		rdata1ID, rdata2ID, wmemdataID, wmemdataEX, rdata1EX, rdata2EX, wdataEX, wdataMEM, wdataWB;
	wire [`XLEN-1:0]		wdata0MEM, wdatawordMEM, wdatahalfMEM, wdatabyteMEM, memhalfMEM, membyteMEM;
	wire [`RFIDX_WIDTH-1:0]	waddrID, waddrEX, waddrMEM, waddrWB;
	wire [`ADDR_SIZE-1:0]   pcID, pcEX;
	wire 					jalEX, jalMEM;
	wire [`XLEN-1:0]        a, b, aluoutEX;
	wire [1:0]				alusrcaEX;
	wire 					alusrcbEX;
	wire [3:0]              aluctrlEX, ampEX;
	wire [1:0] 				lwhbEX, swhbEX, lwhbMEM, swhbMEM; 
	wire 					memwriteEX, memtoregEX, regwriteEX, lunsignedEX;
	wire                    memtoregMEM, lunsignedMEM, regwriteMEM;
	wire 					memwriteWB, memtoregWB, lusignedWB, regwriteWB;

	//处理pc
	mux2 #(32)   mxnewpc(pcplus4IF, pcbranchID, pcsrcID, newpcIF); //pc input 

    pcenr      	 pcr(clk, reset, ~stallIF, newpcIF, pcIF);
    addr_adder   pcadder(pcIF, `ADDR_SIZE'b100, pcplus4IF);

	//IF阶段（instr已读出）
	//IF/ID流水线寄存器
	flopenrc #(`ADDR_SIZE)  flopenrc1IFID(clk, reset, ~stallID, flushID, instrIF, instrID); // instruction
	flopenrc #(`ADDR_SIZE)	flopenrc2IFID(clk, reset, ~stallID, flushID, pcIF, pcID); // pc
	flopenrc #(`ADDR_SIZE)	flopenrc3IFID(clk, reset, ~stallID, flushID, pcplus4IF, pcplus4ID); // pc+4

	//ID阶段
	//decode
	assign  opID		= instrIF[6:0];
	assign  rdID		= instrIF[11:7];
	assign  funct3ID	= instrIF[14:12];
	assign  rs1ID		= instrIF[19:15];
	assign  rs2ID   	= instrIF[24:20];
	assign  funct7ID	= instrIF[31:25];
	assign  immID		= instrIF[31:20];

	// register file
	regfile rf(clk, rs1ID, rs2ID, rdata1ID, rdata2ID, regwriteWB, waddrWB, wdataWB, pcMEM);

	// immediate generation
	wire [11:0]				iimmID  = instrIF[31:20];
	wire [11:0]				simmID	= {instrIF[31:25], instrIF[11:7]};
	wire [11:0]  			bimmID	= {instrIF[31], instrIF[7], instrIF[30:25], instrIF[11:8]};
	wire [19:0]				uimmID	= instrIF[31:12];
	wire [19:0]  			jimmID	= {instrIF[31], instrIF[19:12], instrIF[20], instrIF[30:21]};
	imm 		 im(iimmID, simmID, bimmID, uimmID, jimmID, immctrlID, immoutID);

	sl1          pcsft(immoutID, shftimmID);
	mux2 #(32)   mxbase(pcID, rdata1ID, jalr, base); //adder input a
	mux2 #(32)   mxoffset(shftimmID, immoutID, jalr, offset); //adder input b
	assign  offsetID  = {offset[31:1], offset[0] & !jalr};
	addr_adder   pcjal(base, offsetID, pcbranchID);

	assign       waddrID = rdID;
	assign       wmemdataID = rdata2ID;

	mux2 #(5)    mxshamt(rdata2ID[4:0], instrID[24:20], itype, shamtID); //alu input shamt

	//ID/EX流水线寄存器
	floprc #(1)           floprc1IDEX(clk, reset, flushEX, memwriteID, memwriteEX);
	floprc #(1)           floprc2IDEX(clk, reset, flushEX, memtoregID, memtoregEX);
	floprc #(1)           floprc3IDEX(clk, reset, flushEX, regwriteID, regwriteEX);
	floprc #(2)           floprc4IDEX(clk, reset, flushEX, lwhbID, lwhbEX);
	floprc #(2)           floprc5IDEX(clk, reset, flushEX, swhbID, swhbEX);
	floprc #(2)           floprc6IDEX(clk, reset, flushEX, alusrcaID, alusrcaEX);
	floprc #(1)           floprc7IDEX(clk, reset, flushEX, alusrcbID, alusrcbEX);
	floprc #(4)           floprc8IDEX(clk, reset, flushEX, aluctrlID, aluctrlEX);
	floprc #(1)           floprc9IDEX(clk, reset, flushEX, jalID, jalEX);
	floprc #(1)           floprc16IDEX(clk, reset, flushEX, lunsignedID, lunsignedEX);
	floprc #(32)          floprc10IDEX(clk, reset, flushEX, pcID, pcEX);
	floprc #(32)          floprc11IDEX(clk, reset, flushEX, immoutID, immoutEX);
	floprc #(5)           floprc12IDEX(clk, reset, flushEX, shamtID, shamtEX);
	floprc #(32)          floprc13IDEX(clk, reset, flushEX, pcplus4ID, pcplus4EX);
	floprc #(32)          floprc14IDEX(clk, reset, flushEX, wmemdataID, wmemdataEX);
	floprc #(32)          floprc15IDEX(clk, reset, flushEX, waddrID, waddrEX);
	floprc #(32)          floprc17IDEX(clk, reset, flushEX, rdata1ID, rdata1EX);
	floprc #(32)          floprc18IDEX(clk, reset, flushEX, rdata2ID, rdata2EX);
	//floprc #(4)           floprc19IDEX(clk, reset, flushEX, ampID, ampEX);

	//EX阶段
	mux3 #(32)   mxa(rdata1EX, 0, pcID, alusrcaEX, a); //alu input a
	mux2 #(32)   mxb(rdata2EX, immoutEX, alusrcbEX, b); //alu input b
	// alu
	alu 	alu(a, b, shamtID, aluctrlEX, aluoutEX, overflow, zero, lt, ge);

	//EX/MEM流水线寄存器
	floprc #(1)           floprc1EXMEM(clk, reset, flushMEM, memwriteEX, memwriteMEM);
	floprc #(1)           floprc2EXMEM(clk, reset, flushMEM, memtoregEX, memtoregMEM);
	floprc #(1)           floprc3EXMEM(clk, reset, flushMEM, jalEX, jalMEM);
	floprc #(2)           floprc4EXMEM(clk, reset, flushMEM, lwhbEX, lwhbMEM);
	floprc #(2)           floprc5EXMEM(clk, reset, flushMEM, swhbEX, swhbMEM);
	floprc #(1)           floprc6EXMEM(clk, reset, flushMEM, regwriteEX, regwriteMEM);
	floprc #(1)           floprc12EXMEM(clk, reset, flushMEM, lunsignedEX, lunsignedMEM);
	floprc #(32)		  floprc7EXMEM(clk, reset, flushMEM, aluoutEX, aluoutMEM);
	floprc #(32)		  floprc8EXMEM(clk, reset, flushMEM, wmemdataEX, wmemdataMEM);
	floprc #(32)          floprc9EXMEM(clk, reset, flushMEM, waddrEX, waddrMEM);
	floprc #(32)          floprc10EXMEM(clk, reset, flushMEM, pcEX, pcMEM);
	floprc #(32)          floprc11EXMEM(clk, reset, flushMEM, pcplus4EX, pcplus4MEM);
	//floprc #(4)           floprc13EXMEM(clk, reset, flushMEM, ampEX, ampMEM);

	//MEM阶段	
	mux2 #(32)   mxwdata(aluoutMEM, readdataMEM, memtoregMEM, wdata0MEM); //wdata input
	addr_adder   pcadder1(pcplus4MEM, `ADDR_SIZE'b100, pcplus4IN);
	mux2 #(32)   mxwdataword(wdata0MEM, pcplus4IN, jalMEM, wdatawordMEM); //wdata input pcplus4IF

	//处理lwhb,swhb
	ampattern ampattern(aluoutMEM[1:0], swhbMEM, ampMEM);

	//处理halfword
	mux2 #(16)   mxlh(wdatawordMEM[15:0], wdatawordMEM[31:16], aluoutMEM[1], wdatahalfMEM[15:0]);
	wire [`XLEN-1:0]         signedhalfMEM = {{16{wdatahalfMEM[15]}},wdatahalfMEM[15:0]};
	wire [`XLEN-1:0]         unsignedhalfMEM = {16'b0,{wdatahalfMEM[15:0]}};
	mux2 #(32)   mxlhsign(signedhalfMEM, unsignedhalfMEM, lunsignedMEM, memhalfMEM);
	//处理byte
	mux4 #(8)    mxlb(wdatawordMEM[7:0], wdatawordMEM[15:8], wdatawordMEM[23:16], wdatawordMEM[31:24], aluoutMEM[1:0], wdatabyteMEM[7:0]);
	wire [`XLEN-1:0]         signedbyteMEM = {{24{wdatabyteMEM[7]}},wdatabyteMEM[7:0]};
	wire [`XLEN-1:0]         unsignedbyteMEM = {24'b0,{wdatabyteMEM[7:0]}};
	mux2 #(32)   mxlbsign(signedbyteMEM, unsignedbyteMEM, lunsignedMEM, membyteMEM);

	mux3 #(32)   mxlwhb(wdatawordMEM, memhalfMEM, membyteMEM, lwhbMEM, wdataMEM);

	//MEM/WB流水线寄存器
	floprc #(1)           floprc1MEMWB(clk, reset, flushWB, memwriteMEM, memwriteWB);
	floprc #(1)           floprc2MEMWB(clk, reset, flushWB, memtoregMEM, memtoregWB);
	floprc #(1)           floprc3MEMWB(clk, reset, flushWB, regwriteMEM, regwriteWB);
	floprc #(32)		  floprc4MEMWB(clk, reset, flushWB, wdataMEM, wdataWB);
	floprc #(32)          floprc5MEMWB(clk, reset, flushWB, waddrMEM, waddrWB);
	floprc #(32)          floprc6MEMWB(clk, reset, flushWB, pcMEM, pcWB);

	//cmp     branchcmp(rdata1, rdata2, bunsigned, zero, lt);

	//WB阶段实现写寄存器regfile rf(clk, rs1ID, rs2ID, rdata1ID, rdata2ID, regwriteWB, waddrWB, wdataWB, pcWB);

	
  	always @(negedge clk)
        begin
        $display("pcsrc = %h", pcsrcID);
        end
endmodule